<?php

namespace Drupal\Tests\podtoo\Functional;

use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;

/**
 * Tests for PodToo embed formatter.
 *
 * @group podtoo
 */
class PodTooEmbedFormatterTest extends BrowserTestBase {

  use MediaTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'podtoo',
    'media',
    'link',
  ];

  /**
   * {@inheritDoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Setup standalone media urls from the settings.
    $this->config('media.settings')->set('standalone_url', TRUE)->save();
    $this->refreshVariables();
    // Rebuild routes.
    \Drupal::service('router.builder')->rebuild();

    // Create an admin user with permissions to administer and create media.
    $account = $this->drupalCreateUser([
      // Media entity permissions.
      'view media',
      'create media',
      'update media',
      'update any media',
      'delete media',
      'delete any media',
    ]);

    // Login the user.
    $this->drupalLogin($account);
  }

  /**
   * Tests adding and editing a PodToo embed formatter.
   */
  public function testPodTooEmbedFormatter() {
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface */
    $entity_display_repository = \Drupal::service('entity_display.repository');

    $media_type = $this->createMediaType('podtoo', ['id' => 'podtoo']);

    $source_field = $media_type->getSource()->getSourceFieldDefinition($media_type);
    $this->assertSame('field_media_podtoo', $source_field->getName());
    $this->assertSame('string', $source_field->getType());

    // Set form and view displays.
    \Drupal::service('entity_display.repository')->getFormDisplay('media', $media_type->id(), 'default')
      ->setComponent('field_media_podtoo', [
        'type' => 'string_textfield',
      ])
      ->save();

    \Drupal::service('entity_display.repository')->getViewDisplay('media', $media_type->id(), 'full')
      ->setComponent('field_media_podtoo', [
        'type' => 'podtoo_embed',
      ])
      ->save();

    // Create a PodToo media entity.
    $this->drupalGet('media/add/' . $media_type->id());

    $page = $this->getSession()->getPage();
    $page->fillField('name[0][value]', 'PodToo');
    $page->fillField('field_media_podtoo[0][value]', 'https://podtoo.com/pooriaputak/laelaha');
    $page->pressButton('Save');

    // Assert "has been created" text.
    $assert = $this->assertSession();
    $assert->pageTextContains('has been created');

    // Get to the media page.
    $medias = \Drupal::entityTypeManager()->getStorage('media')->loadByProperties([
      'name' => 'PodToo',
    ]);
    /** @var \Drupal\media\MediaInterface */
    $media = reset($medias);
    $this->drupalGet(Url::fromRoute('entity.media.canonical', ['media' => $media->id()])->toString());
    $assert->statusCodeEquals(200);

    // Assert that the formatter exists on this page.
    $assert->elementExists('css', 'iframe[src*="podtoo"]');
  }

}
